
#include "Calc.h"
#include <string.h>


// Constructor
//
CCalcError::CCalcError(const char *msg, CCalcObject *parent) : CCalcObject(parent)
{
	type = ERROR;

	if (strlen(msg) > ERROR_MSG_LEN -1)
	{
		strncpy(value, msg, ERROR_MSG_LEN -1);
		value[ERROR_MSG_LEN -1] = 0;
	}
	else
		strcpy(value, msg);
}


// Destructor
//
CCalcError::~CCalcError()
{

}


// Execute an instruction
//
CCalcObject *CCalcError::exec(CALC_OP opcode)
{
	CCalcObject *new_top = NULL;

	switch (opcode)
	{
		case DUP:
		{
			new_top = op_dup();
			break;
		}

		case POP:
		{
			new_top = next;
			delete this;
			break;
		}

		case CLR:
			if (next != NULL)
				new_top = next->exec(CLR);
			else
				new_top = NULL;
			delete this;
			break;

		case SWAP:
			if (next == NULL)
			{
				new_top = new CCalcError(getErrorMsg(1), this);
				if (new_top == NULL)
				{
					error_callback("Memory allocation failure.");
					new_top = this;
				}
			}
			else
			{
				new_top = next;
				next = next->next;
				new_top->next = this;
			}
			break;

		case MSTD:
			mode.format = STD;
			new_top = this;
			break;

		case MFIX:
		case MSCI:
		case MENG:
			new_top = new CCalcError(getErrorMsg(2), this);
			break;

		default:
		{
			new_top = new CCalcError(getErrorMsg(0), this);
			if (new_top == NULL)
			{
				error_callback("Memory allocation failure.");
				new_top = this;
			}
			break;
		}

	}

	return new_top;
}


// Returns a string representation of the object's data value
//
const char *CCalcError::getString()
{
	static char repr[STRING_LEN +1];

	strncpy(repr, value, STRING_LEN);
	repr[STRING_LEN] = 0;

	return repr;
}


// Returns a single character to signify the object's type
//
const char CCalcError::getType()
{
	return 'E';
}


// -----------------------------------------------------------
// OPCODE EXECUTION IMPLEMENTATION
// -----------------------------------------------------------

// Duplicates this object and places it into the stack.
//
CCalcObject *CCalcError::op_dup()
{
	CCalcObject *prev = new CCalcError(value, this);
	if (prev == NULL)
	{
		error_callback("Memory allocation failure.");
		prev = this;
	}

	return prev;
}


