#include "CalcMemory.h"
#include "Calc.h"

#include <string.h>
#include <stdio.h>


CCalcVariable::CCalcVariable(const char *name, CCalcObject *object, CCalcVariable *prev)
{
	next = NULL;

	if (strlen(name) > MAX_VAR_NAME)
	{
		strncpy(this->name, name, MAX_VAR_NAME);
		this->name[MAX_VAR_NAME] = 0;
	}
	else
		strcpy(this->name, name);

	this->prev = prev;

	if (prev != NULL)
	{
		if (prev->next != NULL)
			next = prev->next;
		prev->next = this;
	}

	this->object = object;
	object->next = NULL;
}


CCalcVariable::~CCalcVariable()
{
	if (object != NULL)
		delete object;
}


void CCalcVariable::newObject(CCalcObject *object)
{
	if (this->object != NULL)
		delete this->object;
	this->object = object;
	object->next = NULL;
}


CCalcMemory::CCalcMemory()
{
	first = last = NULL;
	numVars = 0;
	lastIndex = -1;
}


CCalcMemory::~CCalcMemory()
{
	CCalcVariable *next;

	while (first != NULL)
	{
		next = first->next;
		delete first;
		first = next;
	}
}


CCalcObject *CCalcMemory::stoVariable(const char *name, CCalcObject *object)
{
	CCalcVariable *var = NULL;
	CCalcObject *rtn = NULL;
	
	// Check for an existing variable
	//
	var = findVariable(name);

	rtn = object->next;

	if (var == NULL)
	{
		var = new CCalcVariable(name, object, last);
		if (var == NULL)
		{
			error_callback("Memory allocation failure.");
			rtn = object;
		}
		else
		{
			if (first == NULL)
				first = var;
			if (last != NULL)
				last->next = var;
			last = var;
			numVars++;
		}
	}
	else
	{
		var->newObject(object);
	}

	return rtn;
}


CCalcObject *CCalcMemory::rclVariable(const char *name, CCalcObject *parent)
{
	CCalcVariable *ptr = NULL;
	CCalcObject *rtn = NULL;

	ptr = findVariable(name);
	if (ptr == NULL)
	{
		rtn = new CCalcError(getErrorMsg(6), parent);
		if (rtn == NULL)
		{
			error_callback("Memory allocation failure.");
			rtn = parent;
		}
	}
	else
	{
		rtn = ptr->object->exec(DUP);
		if (rtn != NULL)
			rtn->next = parent;
	}
	return rtn;
}


CCalcObject *CCalcMemory::delVariable(const char *name, CCalcObject *parent)
{
	CCalcVariable *ptr = NULL;
	CCalcObject *rtn = NULL;

	ptr = findVariable(name);
	if (ptr == NULL)
	{
		rtn = new CCalcError(getErrorMsg(6), parent);
		if (rtn == NULL)
		{
			error_callback("Memory allocation failure.");
			rtn = parent;
		}
	}
	else
	{
		rtn = parent;

		if (ptr->prev != NULL)
			ptr->prev->next = ptr->next;
		if (ptr->next != NULL)
			ptr->next->prev = ptr->prev;
		if (ptr == first)
			first = ptr->next;
		if (ptr == last)
			last = ptr->prev;
		delete ptr;
		numVars--;
	}
	return rtn;
}


long CCalcMemory::getNumVars()
{
	return numVars;
}


const char *CCalcMemory::getVarName(long index)
{
	static CCalcVariable *ptr = NULL;
	static char name[32];
	long i = lastIndex +1;

	if (index == -1)
	{
		index = lastIndex +1;
		if (ptr != NULL)
			ptr = ptr->next;
	}
	else
	{
		ptr = first;
		for (i=0; i<index; i++)
			if (ptr != NULL)
				ptr = ptr->next;
	}

	if (ptr != NULL)
	{
		strncpy(name, ptr->name, MAX_VAR_NAME);
		name[MAX_VAR_NAME] = 0;
	}
	else
		strcpy(name, "");

	return name;
}


CCalcVariable *CCalcMemory::findVariable(const char *name)
{
	CCalcVariable *ptr = first;
	bool found = false;
	char searchName[MAX_VAR_NAME +1];

	if (strlen(name) > MAX_VAR_NAME)
	{
		strncpy(searchName, name, MAX_VAR_NAME);
		searchName[MAX_VAR_NAME] = 0;
	}
	else
		strcpy(searchName, name);

	while (!found && (ptr != NULL))
	{
		if (!strcmp(ptr->name, searchName))
			found = true;
		else
			ptr = ptr->next;
	}
	return ptr;
}

