
#include "Calc.h"
#include "CalcStack.h"

#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>

static char error_msg[65];
static void (*ext_error_callback)(void);

CCalcMemory mem;


CCalcStack::CCalcStack(void (*error_fn)(void))
{
	base = NULL;
	::error_callback = &callError;
	::ext_error_callback = error_fn;
	error_msg = ::error_msg;

	// Set default calculator modes
	//
	mode = &::mode;
	mode->drg		= DEG;
	mode->format	= STD;
	mode->decimals	= 12;
	mode->base		= 10;
	mode->bindigits	= 16;
	mode->sign10	= true;

	printf("%d\n", sizeof(CCalcDouble));
	printf("%d\n", sizeof(CCalcInteger));
	printf("%d\n", sizeof(CCalcString));
	printf("%d\n", sizeof(CCalcError));
}


CCalcStack::~CCalcStack()
{
	if (base != NULL)
		base = base->exec(CLR);
}


void CCalcStack::callError(const char *msg)
{
	strncpy(::error_msg, msg, 64);
	ext_error_callback();
}


// Push an object onto the stack.
//
void CCalcStack::push(CCalcObject *object)
{
	object->next = base;
	base = object;
}


// This function parses an input string and determines what type of
// stack object to place it in.  Then it allocates and returns the
// object.
//
void CCalcStack::parseNewObject(const char *expression)
{
	CCalcObject *new_object = NULL;
	long i;
	CALC_OBJECT_DESCR type = INTEGER;
	char *stop_char = NULL;
	int decimals = 0;
	int exps = 0;
	bool expBegin = false;
	char new_str[ERROR_MSG_LEN];

	if (expression == NULL)
		return;

	size_t len = strlen(expression);
	if (len > ERROR_MSG_LEN)
	{
		type = ERROR;
	}
	else
	{
		for (i=0; i<len; i++)
		{
			if ((i == 0) && (expression[i] == '-'))
			{
			}
			else if ((i == 0) && (expression[i] == '\''))
			{
				type = STRING;
			}
			else
			{
				switch (type)
				{
					case INTEGER:
						if (isIntegerDigit(expression[i]))
							break;
						else if (isFloatDigit(expression[i]))
						{
							type = DOUBLE;
							i = -1;
							break;
						}
						else
						{
							type = ERROR;
							break;
						}

					case DOUBLE:
						if (isFloatDigit(expression[i]) || (expression[i] == '-'))
						{
							if (expression[i] == '.')
							{
								decimals++;
								expBegin = false;
							}
							if (expression[i] == '-')
							{
								if (!expBegin)
									type = ERROR;
								expBegin = false;
							}
							else if ((expression[i] == 'E') || (expression[i] == 'e'))
							{
								expBegin = true;
								exps++;
							}
							else
								expBegin = false;
							break;
						}
						else
						{
							type = ERROR;
							break;
						}

					case ERROR:
						break;

					case STRING:
						break;
				}
				//printf("%c %d\n", expression[i], type);
			}
		}
	}
	if (((exps > 1) || (decimals > 1)) && (type == DOUBLE))
		type = ERROR;
	if ((type == STRING) && (len <= 1))
		type = ERROR;
	if ((type == STRING) && (expression[len -1] != '\''))
		type = ERROR;

	switch (type)
	{
		case INTEGER:
			switch (mode->base)
			{
				case 10:
					if (mode->sign10)
						new_object = new CCalcInteger(strtol(expression, &stop_char, 10), base);
					else
						new_object = new CCalcInteger((long)(strtoul(expression, &stop_char, 10)), base);
					break;

				case 2:
				case 8:
				case 16:

					new_object = new CCalcInteger((long)(strtoul(expression, &stop_char, mode->base)), base);
					break;

				default:
					error_callback("Invalid base (!BODH).");
					new_object = base;
			}
			break;

		case DOUBLE:
			new_object = new CCalcDouble(strtod(expression, &stop_char), base);
			break;

		case ERROR:
			new_object = new CCalcError(getErrorMsg(3), base);
			break;

		case STRING:
			strncpy(new_str, &expression[1], len -2);
			new_str[len -2] = 0;
			new_object = new CCalcString(new_str, base);
			break;

		default:
			error_callback("Invalid object type parsed.");
			new_object = base;
			break;
	}

	base = new_object;
	//printf("%d\n", sizeof(*base));
}


const char &CCalcStack::pop()
{
	static char repr[STRING_LEN +1];

	if (base == NULL)
		sprintf(repr, "*(empty)*");
	strcpy(repr, base->getString());
	
	base = base->exec(POP);

	return *repr;
}


void CCalcStack::exec(CALC_OP opcode)
{
	if (base == NULL)
		push(new CCalcError(getErrorMsg(5), base));
	else
		base = base->exec(opcode);
}


// Utility routine to determine if a digit is valid based on the current
// integer base.
bool CCalcStack::isIntegerDigit(const char digit)
{
	switch (mode->base)
	{
		case 16:
			if (isxdigit(digit) != 0)
				return true;
			else
				return false;

		case 10:
			if (isdigit(digit) != 0)
				return true;
			else
				return false;

		case 8:
			if ((isdigit(digit) != 0) && (digit != '8') && (digit != '9'))
				return true;
			else
				return false;

		case 2:
			if ((digit == '0') || (digit == '1'))
				return true;
			else
				return false;

		default:
			error_callback("Invalid base (!BODH).");
			return false;
	}
}


// Utility routine to determine if a digit is a valid floating point digit.
//
bool CCalcStack::isFloatDigit(const char digit)
{
	if (isdigit(digit) || (digit == '.') || (digit == 'E') || (digit == 'e'))
		return true;
	else
		return false;
}
