
#include "Calc.h"
#include <string.h>
#include <stdio.h>


// Constructor
//
CCalcString::CCalcString(const char *val, CCalcObject *parent) : CCalcObject(parent)
{
	type = STRING;

	if (strlen(val) > ERROR_MSG_LEN -1)
	{
		strncpy(value, val, ERROR_MSG_LEN -1);
		value[ERROR_MSG_LEN -1] = 0;
	}
	else
		strcpy(value, val);
}


// Destructor
//
CCalcString::~CCalcString()
{

}


// Execute an instruction
//
CCalcObject *CCalcString::exec(CALC_OP opcode)
{
	CCalcObject *new_top = NULL;

	switch (opcode)
	{
		case DUP:
		{
			new_top = op_dup();
			break;
		}

		case POP:
		{
			new_top = next;
			delete this;
			break;
		}

		case CLR:
			if (next != NULL)
				new_top = next->exec(CLR);
			else
				new_top = NULL;
			delete this;
			break;

		case SWAP:
			if (next == NULL)
			{
				new_top = new CCalcError(getErrorMsg(1), this);
				if (new_top == NULL)
				{
					error_callback("Memory allocation failure.");
					new_top = this;
				}
			}
			else
			{
				new_top = next;
				next = next->next;
				new_top->next = this;
			}
			break;

		case MSTD:
			mode.format = STD;
			new_top = this;
			break;

		case MFIX:
		case MSCI:
		case MENG:
			new_top = new CCalcError(getErrorMsg(2), this);
			break;

		case STO:
			if (next == NULL)
				new_top = new CCalcError(getErrorMsg(1), this);
			else
			{
				new_top = mem.stoVariable(value, next);
				delete this;
			}
			break;

		case RCL:
			new_top = mem.rclVariable(value, next);
			if (new_top == next)
				new_top = this;
			else
				delete this;
			break;

		default:
		{
			new_top = new CCalcError(getErrorMsg(2), this);
			if (new_top == NULL)
			{
				error_callback("Memory allocation failure.");
				new_top = this;
			}
			break;
		}

	}

	return new_top;
}


// Returns a string representation of the object's data value
//
const char *CCalcString::getString()
{
	static char repr[STRING_LEN +2];

	sprintf(repr, "'%s'", value);

	return repr;
}


// Returns a single character to signify the object's type
//
const char CCalcString::getType()
{
	return 'S';
}


// -----------------------------------------------------------
// OPCODE EXECUTION IMPLEMENTATION
// -----------------------------------------------------------

// Duplicates this object and places it into the stack.
//
CCalcObject *CCalcString::op_dup()
{
	CCalcObject *prev = new CCalcString(value, this);
	if (prev == NULL)
	{
		error_callback("Memory allocation failure.");
		prev = this;
	}

	return prev;
}


