#include "CalcWindow.h"
#include "CalcButton.h"

#include <qmessagebox.h>
#include <qfont.h>
#include <qpe/config.h>

extern CCalcObject *parseNewObject(CCalcObject *parent, const char *expression);


static const char * test_xpm[] = {
	"16 16 3 1",
	"       c None",
	".      c #FFFFFF",
	"+      c #000000",
	".+++++++++++++++",
	"++++++++++++++++",
	"++++++++++++++++",
	"++++++++++++++++",
	"++++++++++++++++",
	"++++++++++++++++",
	"++++++++++++++++",
	"++++++++++++++++",
	"++++++++++++++++",
	"++++++++++++++++",
	"++++++++++++++++",
	"++++++++++++++++",
	".+++++++++++++++",
	".+++++++++++++++",
	"...+++++++++++..",
	"....+++++++++..."};

static QCalcWindow *mainWindow;

QCalcWindow::QCalcWindow(QWidget *parent, const char *name, WFlags fl) : QWidget(parent, name, fl)
{
	QPushButton *b;
	QString temp;
	inverse = false;
	CalcConfig = NULL;
	varIndex = 0;
	::mainWindow = this;
	stack = new CCalcStack(&error_callback);

	QPixmap *pm = new QPixmap(test_xpm);

	setCaption(QString("SlideRuleZ"));

	QFont listFont("fixed", 12, QFont::Bold);
	listFont.setFixedPitch(true);
	setFont( listFont );

	QVBoxLayout *layout0 = new QVBoxLayout(this);
	stackList = new QListBox(this);
	stackList->setVScrollBarMode(QScrollView::AlwaysOff);
	temp.sprintf("%d:%c %32s", 1, ' ', "setup");
	stackList->insertItem(temp, 0);
	stackList->insertItem(temp, 0);
	stackList->insertItem(temp, 0);
	stackList->insertItem(temp, 0);
	stackList->setMinimumSize(stackList->sizeHint());
	layout0->addWidget(stackList);

	QHBoxLayout *layout3 = new QHBoxLayout(this);
	layout0->addLayout(layout3);

	annuncMode= new QLabel(this);
	annuncMode->setFont( QFont("fixed", 10) );
	annuncMode->setText("DEG  DEC STD");
	annuncMode->setFrameShape(QFrame::Panel);
	annuncMode->setFrameShadow(QFrame::Sunken);
	layout3->addWidget(annuncMode);
	
	stackTop = new QLineEdit(this);
	stackTop->setAlignment(Qt::AlignRight);
	stackTop->setFont( QFont("helvetica", 14, QFont::Black) );
	layout3->addWidget(stackTop);

	// Upper buttons
	// 
	QGridLayout *layout2 = new QGridLayout(this, 4, 6, 1);
	layout0->addLayout(layout2);

	pButton[0] = new QCalcButton("", this);
	layout2->addWidget(pButton[0], 0, 0);
	QObject::connect(pButton[0], SIGNAL( funcOp(const QString &, const CALC_OP) ),
			this, SLOT (funcButton(const QString &, const CALC_OP) ));
	vButton[0] = new QCalcButton("", this);
	layout2->addWidget(vButton[0], 0, 1);
	QObject::connect(vButton[0], SIGNAL( funcName(const QString &) ),
			this, SLOT (varButton(const QString &) ));
	vButton[1] = new QCalcButton("", this);
	layout2->addWidget(vButton[1], 0, 2);
	QObject::connect(vButton[1], SIGNAL( funcName(const QString &) ),
			this, SLOT (varButton(const QString &) ));
	vButton[2] = new QCalcButton("", this);
	layout2->addWidget(vButton[2], 0, 3);
	QObject::connect(vButton[2], SIGNAL( funcName(const QString &) ),
			this, SLOT (varButton(const QString &) ));
	vButton[3] = new QCalcButton("", this);
	layout2->addWidget(vButton[3], 0, 4);
	QObject::connect(vButton[3], SIGNAL( funcName(const QString &) ),
			this, SLOT (varButton(const QString &) ));
	vButton[4] = new QCalcButton("", this);
	layout2->addWidget(vButton[4], 0, 5);
	QObject::connect(vButton[4], SIGNAL( funcName(const QString &) ),
			this, SLOT (varButton(const QString &) ));

	pButton[1] = new QCalcButton("", this);
	layout2->addWidget(pButton[1], 1, 0);
	QObject::connect(pButton[1], SIGNAL( funcOp(const QString &, const CALC_OP) ),
			this, SLOT (funcButton(const QString &, const CALC_OP) ));
	pButton[2] = new QCalcButton("", this);
	layout2->addWidget(pButton[2], 1, 1);
	QObject::connect(pButton[2], SIGNAL( funcOp(const QString &, const CALC_OP) ),
			this, SLOT (funcButton(const QString &, const CALC_OP) ));
	pButton[3] = new QCalcButton("", this);
	layout2->addWidget(pButton[3], 1, 2);
	QObject::connect(pButton[3], SIGNAL( funcOp(const QString &, const CALC_OP) ),
			this, SLOT (funcButton(const QString &, const CALC_OP) ));
	pButton[4] = new QCalcButton("", this);
	layout2->addWidget(pButton[4], 1, 3);
	QObject::connect(pButton[4], SIGNAL( funcOp(const QString &, const CALC_OP) ),
			this, SLOT (funcButton(const QString &, const CALC_OP) ));
	b = new QCalcButton("PREV", this);
	layout2->addWidget(b, 1, 4);
	QObject::connect(b, SIGNAL( funcName(const QString &) ),
			this, SLOT (dirButton(const QString &) ));
	b = new QCalcButton("NEXT", this);
	layout2->addWidget(b, 1, 5);
	QObject::connect(b, SIGNAL( funcName(const QString &) ),
			this, SLOT (dirButton(const QString &) ));
	
	pButton[5] = new QCalcButton("", this);
	layout2->addWidget(pButton[5], 2, 0);
	QObject::connect(pButton[5], SIGNAL( funcOp(const QString &, const CALC_OP) ),
			this, SLOT (funcButton(const QString &, const CALC_OP) ));
	pButton[6] = new QCalcButton("", this);
	layout2->addWidget(pButton[6], 2, 1);
	QObject::connect(pButton[6], SIGNAL( funcOp(const QString &, const CALC_OP) ),
			this, SLOT (funcButton(const QString &, const CALC_OP) ));
	pButton[7] = new QCalcButton("", this);
	layout2->addWidget(pButton[7], 2, 2);
	QObject::connect(pButton[7], SIGNAL( funcOp(const QString &, const CALC_OP) ),
			this, SLOT (funcButton(const QString &, const CALC_OP) ));
	pButton[8] = new QCalcButton("", this);
	layout2->addWidget(pButton[8], 2, 3);
	QObject::connect(pButton[8], SIGNAL( funcOp(const QString &, const CALC_OP) ),
			this, SLOT (funcButton(const QString &, const CALC_OP) ));
	pButton[9] = new QCalcButton("", this);
	layout2->addWidget(pButton[9], 2, 4);
	QObject::connect(pButton[9], SIGNAL( funcOp(const QString &, const CALC_OP) ),
			this, SLOT (funcButton(const QString &, const CALC_OP) ));
	b = new QCalcButton("SWAP", this, SWAP);
	layout2->addWidget(b, 2, 5);
	QObject::connect(b, SIGNAL( funcOp(const QString &, const CALC_OP) ),
			this, SLOT (funcButton(const QString &, const CALC_OP) ));

	b = new QCalcButton("ENTER", this);
	layout2->addMultiCellWidget(b, 3, 3, 0, 1);
	QObject::connect(b, SIGNAL( funcName(const QString &) ),
			this, SLOT (editButton(const QString &) ));
	pButton[10] = new QCalcButton("", this);
	layout2->addWidget(pButton[10], 3, 2);
	QObject::connect(pButton[10], SIGNAL( funcOp(const QString &, const CALC_OP) ),
			this, SLOT (funcButton(const QString &, const CALC_OP) ));
	b = new QCalcButton("EEX", this);
	layout2->addWidget(b, 3, 3);
	QObject::connect(b, SIGNAL( funcName(const QString &) ),
			this, SLOT (editButton(const QString &) ));
	b = new QCalcButton("EDIT", this);
	layout2->addWidget(b, 3, 4);
	QObject::connect(b, SIGNAL( funcName(const QString &) ),
			this, SLOT (editButton(const QString &) ));
	eButton = new QCalcButton("<-", this);
	layout2->addWidget(eButton, 3, 5);
	QObject::connect(eButton, SIGNAL( funcName(const QString &) ),
			this, SLOT (editButton(const QString &) ));

	// Lower buttons
	//
	QGridLayout *layout1 = new QGridLayout(this, 4, 5, 1);
	layout0->addLayout(layout1);

	b = new QCalcButton("INV", this);
	layout1->addWidget(b, 0, 0);
	QObject::connect(b, SIGNAL( funcName(const QString &) ),
			this, SLOT (modeButton(const QString &) ));

	b = new QCalcButton("7", this);
	//b->setFont( QFont("helvetica", 10, QFont::Bold) );
	layout1->addWidget(b, 0, 1);
	QObject::connect(b, SIGNAL( funcName(const QString &) ),
			this, SLOT (editButton(const QString &) ));
	b = new QCalcButton("8", this);
	////b->setFont( QFont("helvetica", 10, QFont::Bold) );
	layout1->addWidget(b, 0, 2);
	QObject::connect(b, SIGNAL( funcName(const QString &) ),
			this, SLOT (editButton(const QString &) ));
	b = new QCalcButton("9", this);
	//b->setFont( QFont("helvetica", 10, QFont::Bold) );
	layout1->addWidget(b, 0, 3);
	QObject::connect(b, SIGNAL( funcName(const QString &) ),
			this, SLOT (editButton(const QString &) ));
	b = new QCalcButton("/", this, DIV);
	//b->setFont( QFont("helvetica", 10, QFont::Bold) );
	layout1->addWidget(b, 0, 4);
	QObject::connect(b, SIGNAL( funcOp(const QString &, const CALC_OP) ),
			this, SLOT (funcButton(const QString &, const CALC_OP) ));
	
	b = new QCalcButton("MATH", this);
	layout1->addWidget(b, 1, 0);
	QObject::connect(b, SIGNAL( funcName(const QString &) ),
			this, SLOT (modeButton(const QString &) ));
	b = new QCalcButton("4", this);
	//b->setFont( QFont("helvetica", 10, QFont::Bold) );
	layout1->addWidget(b, 1, 1);
	QObject::connect(b, SIGNAL( funcName(const QString &) ),
			this, SLOT (editButton(const QString &) ));
	b = new QCalcButton("5", this);
	//b->setFont( QFont("helvetica", 10, QFont::Bold) );
	layout1->addWidget(b, 1, 2);
	QObject::connect(b, SIGNAL( funcName(const QString &) ),
			this, SLOT (editButton(const QString &) ));
	b = new QCalcButton("6", this);
	//b->setFont( QFont("helvetica", 10, QFont::Bold) );
	layout1->addWidget(b, 1, 3);
	QObject::connect(b, SIGNAL( funcName(const QString &) ),
			this, SLOT (editButton(const QString &) ));
	b = new QCalcButton("x", this, MUL);
	//b->setFont( QFont("helvetica", 10, QFont::Bold) );
	layout1->addWidget(b, 1, 4);
	QObject::connect(b, SIGNAL( funcOp(const QString &, const CALC_OP) ),
			this, SLOT (funcButton(const QString &, const CALC_OP) ));

	b = new QCalcButton("BIN", this);
	layout1->addWidget(b, 2, 0);
	QObject::connect(b, SIGNAL( funcName(const QString &) ),
			this, SLOT (modeButton(const QString &) ));
	b = new QCalcButton("1", this);
	//b->setFont( QFont("helvetica", 10, QFont::Bold) );
	layout1->addWidget(b, 2, 1);
	QObject::connect(b, SIGNAL( funcName(const QString &) ),
			this, SLOT (editButton(const QString &) ));
	b = new QCalcButton("2", this);
	//b->setFont( QFont("helvetica", 10, QFont::Bold) );
	layout1->addWidget(b, 2, 2);
	QObject::connect(b, SIGNAL( funcName(const QString &) ),
			this, SLOT (editButton(const QString &) ));
	b = new QCalcButton("3", this);
	//b->setFont( QFont("helvetica", 10, QFont::Bold) );
	layout1->addWidget(b, 2, 3);
	QObject::connect(b, SIGNAL( funcName(const QString &) ),
			this, SLOT (editButton(const QString &) ));
	b = new QCalcButton("-", this, SUB);
	//b->setFont( QFont("helvetica", 10, QFont::Bold) );
	layout1->addWidget(b, 2, 4);
	QObject::connect(b, SIGNAL( funcOp(const QString &, const CALC_OP) ),
			this, SLOT (funcButton(const QString &, const CALC_OP) ));

	b = new QCalcButton("MEM", this);
	layout1->addWidget(b, 3, 0);
	QObject::connect(b, SIGNAL( funcName(const QString &) ),
			this, SLOT (modeButton(const QString &) ));
	b = new QCalcButton("0", this);
	//b->setFont( QFont("helvetica", 10, QFont::Bold) );
	layout1->addWidget(b, 3, 1);
	QObject::connect(b, SIGNAL( funcName(const QString &) ),
			this, SLOT (editButton(const QString &) ));
	b = new QCalcButton(".", this);
	//b->setFont( QFont("helvetica", 10, QFont::Bold) );
	layout1->addWidget(b, 3, 2);
	QObject::connect(b, SIGNAL( funcName(const QString &) ),
			this, SLOT (editButton(const QString &) ));
	b = new QCalcButton("CHS", this);
	layout1->addWidget(b, 3, 3);
	QObject::connect(b, SIGNAL( funcName(const QString &) ),
			this, SLOT (editButton(const QString &) ));
	b = new QCalcButton("+", this, ADD);
	//b->setFont( QFont("helvetica", 10, QFont::Bold) );
	layout1->addWidget(b, 3, 4);
	QObject::connect(b, SIGNAL( funcOp(const QString &, const CALC_OP) ),
			this, SLOT (funcButton(const QString &, const CALC_OP) ));

	setButtonMode(MATH);

	layout0->activate();

	stackTop->setFocus();
}


QCalcWindow::~QCalcWindow()
{
	delete stack;
}


// Error callback
//
void QCalcWindow::error_callback(void)
{
	QMessageBox::critical(mainWindow, "SlideRuleZ", mainWindow->stack->error_msg);
}


void QCalcWindow::setButtonMode(BUTTON_MODE mode)
{
	this->mode = mode;

	if (!inverse)
		eButton->setFunc("<-");
	else
		eButton->setFunc("CLR", CLR);

	switch (mode)
	{
		case MATH:
			pButton[0]->setFunc("");
			pButton[1]->setFunc("");
			if (!inverse)
			{
				pButton[2]->setFunc("SIN", SIN);
				pButton[3]->setFunc("COS", COS);
				pButton[4]->setFunc("TAN", TAN);
				pButton[5]->setFunc("LOG", LOG);
				pButton[6]->setFunc("LN", LN);
				pButton[7]->setFunc("SQRT", SQRT);
				pButton[8]->setFunc("y^x", PWR);
			}
			else
			{
				pButton[2]->setFunc("ASIN", ASIN);
				pButton[3]->setFunc("ACOS", ACOS);
				pButton[4]->setFunc("ATAN", ATAN);
				pButton[5]->setFunc("ALOG", ALOG);
				pButton[6]->setFunc("e^x", EXP);
				pButton[7]->setFunc("x^2", SQR);
				pButton[8]->setFunc("ROOT", ROOT);
			}
			pButton[9]->setFunc("1/x");
			pButton[10]->setFunc("DRG");

			updateVars();	
			//vButton[0]->setFunc("");
			//vButton[1]->setFunc("");
			//vButton[2]->setFunc("");
			//vButton[3]->setFunc("");
			//vButton[4]->setFunc("");
			break;
			
		case BIN:
			pButton[0]->setFunc("A");
			pButton[1]->setFunc("BIN");
			pButton[2]->setFunc("OCT");
			pButton[3]->setFunc("DEC");
			pButton[4]->setFunc("HEX");
			pButton[5]->setFunc("");
			pButton[6]->setFunc("");
			pButton[7]->setFunc("");
			pButton[8]->setFunc("");
			pButton[9]->setFunc("");
			pButton[10]->setFunc("INT", INT);

			vButton[0]->setFunc("B");
			vButton[1]->setFunc("C");
			vButton[2]->setFunc("D");
			vButton[3]->setFunc("E");
			vButton[4]->setFunc("F");
			break;

		case MEM:
			pButton[0]->setFunc("");
			pButton[1]->setFunc("'");
			pButton[2]->setFunc("STO", STO);
			pButton[3]->setFunc("RCL", RCL);
			pButton[4]->setFunc("DIR", DIR);
			pButton[5]->setFunc("STD", MSTD);
			pButton[6]->setFunc("FIX", MFIX);
			pButton[7]->setFunc("SCI", MSCI);
			pButton[8]->setFunc("ENG", MENG);
			pButton[9]->setFunc("");
			pButton[10]->setFunc("");

			updateVars();	
			//vButton[0]->setFunc("");
			//vButton[1]->setFunc("");
			//vButton[2]->setFunc("");
			//vButton[3]->setFunc("");
			//vButton[4]->setFunc("");
			break;
	}

	updateStack();
}


void QCalcWindow::modeButton(const QString &btn)
{
	if (btn == "MATH")
		setButtonMode(MATH);
	else if (btn == "BIN")
		setButtonMode(BIN);
	else if (btn == "MEM")
		setButtonMode(MEM);
	else if (btn == "INV")
	{
		inverse = !inverse;
		setButtonMode(mode);
	}
}


// editButton()
//
// Slot to handle button presses that affect the current edit.
//
void QCalcWindow::editButton(const QString &btn)
{
	int i;
	bool found = false;
	bool flt = false;
	int pos;
	
	if (btn == "ENTER")
	{
		if (stackTop->text().length() == 0)
			funcButton(btn, DUP);
		else
		{
			stack->parseNewObject((const char *) stackTop->text());
			updateStack();
			stackTop->clear();
		}
	}
	else if (btn == "EDIT")
	{
		if (stackTop->text().length() == 0)
		{
			if (stack->base == NULL)
				stack->push(new CCalcError(getErrorMsg(5)));
			else
				stackTop->setText(&(stack->pop()));
			updateStack();
		}
		else
		{
		}
			
	}
	else if (btn == "CLR")
	{
		if (stackTop->text().length() == 0)
			funcButton(btn, CLR);
		else
			stackTop->clear();
		inverse = false;
		setButtonMode(mode);
	}
	else if (btn == "<-")
	{
		if (stackTop->text().length() == 0)
		{
			if (stack->base != NULL)
				funcButton(btn, POP);
		}
		else
			stackTop->backspace();
	}
	else if (btn == "EEX")
	{
		stackTop->setText(stackTop->text() +"E");
	}
	else if (btn == "CHS")
	{
		if (stackTop->text().length() == 0)
			funcButton(btn, CHS);
		else
		{
			for (i=0; i<stackTop->text().length(); i++)
			{
				if (stackTop->text()[i] == '.')
					flt = true;
				else if ((stackTop->text()[i] == 'E') && ((stack->mode->base != 16) || (flt)))
				{
					found = true;
					break;
				}
			}
			if (found)
			{
				if (stackTop->text()[i +1] == '-')
					stackTop->setText(stackTop->text().remove(i +1, 1));
				else
					stackTop->setText(stackTop->text().insert(i +1, '-'));
			}
			else
			{
				if (stackTop->text()[0] == '-')
					stackTop->setText(stackTop->text().remove(0, 1));
				else
					stackTop->setText(stackTop->text().insert(0, "-"));
			}
		}
	}
	else if (btn == "'")
	{
		editButton("''");
	}
	else
	{
		pos = stackTop->cursorPosition();
		stackTop->setText(stackTop->text().insert(pos, btn));
		stackTop->setCursorPosition(pos +1);
	}
}


// funcButton()
// 
// Slot to handle function button presses.  Most of the time,
// after pushing any pending edits onto the stack,
// the correct thing to do is to pass the function to the
// RPN engine for execution.
// 
void QCalcWindow::funcButton(const QString &text, const CALC_OP opcode)
{
	QString msg;
	CALC_OP code = opcode;

	// Check for special cases that aren't really functions
	//
	if (text == "")
		return;
	if (text == "'")
	{
		editButton("'");
		return;
	}	
	if ((mode == BIN) && (text == "A"))
	{
		editButton(text);
		return;
	}
	if (text == "DRG")
	{
		if (stack->mode->drg == DEG)
			stack->mode->drg = RAD;
		else if (stack->mode->drg == RAD)
			stack->mode->drg = GRAD;
		else if (stack->mode->drg == GRAD)
			stack->mode->drg = DEG;
		updateStack();
		return;
	}
	if (text == "BIN")
	{
		stack->mode->base = 2;
		updateStack();
		return;
	}
	if (text == "OCT")
	{
		stack->mode->base = 8;
		updateStack();
		return;
	}
	if (text == "DEC")
	{
		stack->mode->base = 10;
		updateStack();
		return;
	}
	if (text == "HEX")
	{
		stack->mode->base= 16;
		updateStack();
		return;
	}


	// Special case functions
	//
	if (text == "1/x")
	{
		stack->parseNewObject("1");
		if ((stackTop->text().length() == 0) && (stack->base != NULL))
			stack->exec(SWAP);
		code = DIV;
	}
		

	// If there is a pending edit, push it onto the stack first.
	//
	if (stackTop->text().length() > 0)
	{
		editButton("ENTER");

		// Make sure it didn't result in an error
		//
		if (stack->base->type == ERROR)
		{
			inverse = false;
			setButtonMode(mode);
			updateStack();
			return;
		}
	}	

	// Most opcodes just pass through to the RPN
	// engine.  Only unusual behavior needs to
	// be coded here.
	//
	switch (code)
	{
		case CLR:
			if (stack->base == NULL)
			{
				inverse = false;
				setButtonMode(mode);
				updateStack();
				return;
			}
			break;

		case NOP:
			break;

		default:
			break;
	}

	// Execute opcode
	//
	stack->exec(code);

	// Clear the inverse if it was set
	// 
	inverse = false;
	setButtonMode(mode);

	updateStack();
	updateVars();
}


void QCalcWindow::varButton(const QString &btn)
{
	QString name;
	int pos;

	if (mode == BIN)
		editButton(btn);
	else
	{
		if (stackTop->text().length() == 0)
		{
			name = "'" +btn +"'";
			stack->parseNewObject(name);
			funcButton("RCL", RCL);
		}
		else
		{
			pos = stackTop->cursorPosition();
			stackTop->setText(stackTop->text().insert(pos, btn));
			stackTop->setCursorPosition(pos +1);
		}
	}
}


void QCalcWindow::dirButton(const QString &btn)
{
	if (btn == "NEXT")
		varIndex = (varIndex +5 >= mem.getNumVars()) ? 0 : varIndex +5;
	if (btn == "PREV")
		varIndex = (varIndex -5 < 0) ? ((long)(mem.getNumVars()/5))*5 : varIndex -5;
	updateVars();
}


void QCalcWindow::updateStack()
{
	QString buffer;
	int i = 0;

	CCalcObject *ptr = stack->base;

	stackList->clear();
	for (i=1; i<=4; i++)
	{
		if (ptr != NULL)
		{
			buffer.sprintf("%d:%c %32s", i, ptr->getType(), ptr->getString());
			ptr = ptr->next;
		}
		else
			buffer.sprintf("%d:%c %32s", i, ' ', "");
		stackList->insertItem(buffer, 0);
	}

	updateMode();
}


void QCalcWindow::updateMode()
{
	QString buffer;

	switch (stack->mode->drg)
	{
		case DEG:
			buffer = "DEG  ";
			break;

		case RAD:
			buffer = "RAD  ";
			break;

		case GRAD:
			buffer = "GRAD ";
			break;
	}

	switch (stack->mode->base)
	{
		case 2:
			buffer += "BIN ";
			break;

		case 8:
			buffer += "OCT ";
			break;

		case 10:
			buffer += "DEC ";
			break;

		case 16:
			buffer += "HEX ";
			break;
	}

	switch (stack->mode->format)
	{
		case STD:
			buffer += "STD";
			break;

		case FIX:
			buffer += "FIX";
			break;

		case SCI:
			buffer += "SCI";
			break;

		case ENG:
			buffer += "ENG";
	}

	annuncMode->setText(buffer);
}


void QCalcWindow::updateVars()
{
	long i;

	if (mode != BIN)
	{
		vButton[0]->setFunc(mem.getVarName(varIndex));
		for (i=1; i<5; i++)
			vButton[i]->setFunc(mem.getVarName());
	}
}


void QCalcWindow::ReadConfig()
{
	long numVars = 0;
	long i;
	QString varName;

	CalcConfig = new Config("sliderulez");
	if (CalcConfig == NULL)
		QMessageBox::critical(mainWindow, "SlideRuleZ", "Memory allocation failure.");
	else
	{
		CalcConfig->setGroup("variables");
		numVars = CalcConfig->readNumEntry("vars", 0);
		for (i=0; i<numVars; i++)
		{
		}
	}
}


void QCalcWindow::PurgeConfig()
{
}


void QCalcWindow::WriteConfig()
{
}


